// FamilyCode10Servlet.java - A demo servlet based on OneWireContainer10.
//
// Copyright (C) 1999-2002  Smart Software Consulting
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
//
// http://www.smartsc.com
//

import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

import javax.servlet.*;
import javax.servlet.http.*;

import com.dalsemi.onewire.OneWireException;
import com.dalsemi.onewire.adapter.DSPortAdapter;
import com.dalsemi.onewire.container.OneWireContainer;
import com.dalsemi.onewire.container.OneWireContainer10;
import com.dalsemi.onewire.utils.Address;

public class FamilyCode10Servlet
extends HttpServlet
{
	public void doGet( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		String adapter = req.getParameter( OneWireServlet.ADAPTER);
		String port = req.getParameter( OneWireServlet.PORT);
		String ibString = req.getParameter( OneWireServlet.DEVICE);

		if( adapter  == null || port == null
		||  ibString == null || ibString.length() != 16)
		{
			throw new ServletException(
				getClass().getName() + ": Invalid parameters.");
		}

		byte[] id = Address.toByteArray( ibString);

		// Do HTTP header stuff
		res.setContentType( "text/html");

		// Get Writer
		PrintWriter pw = res.getWriter();

		// Start HTML
		pw.println( "<html>");
		pw.println( "<head><title>1-Wire Temperature Servlet</title></head>");
		pw.println( "<body>");


		DSPortAdapter pa = null;
		try
		{
			pa = OneWireServlet.getAdapter( adapter, port);
			OneWireContainer10 ib = new OneWireContainer10();
			ib.setupContainer( pa, id);
			sendBody( pw, pa, ib);
		}
		catch( Exception e)
		{
			OneWireServlet.sendAlert( pw, e);
			pw.println( "<P>Error: " + e.getMessage());
		}

		// Finish HTML
		pw.println("</body></html>");
	}

	public void sendBody(
		PrintWriter pw, DSPortAdapter pa, OneWireContainer10 ib)
	throws OneWireException
	{
		if( ib.isPresent())
		{
		    byte[] state = ib.readDevice();
			pw.print( "<H1>Current Temperature of 1-Wire temperature device ");
			pw.print( ib.getAddressAsString());
			pw.println( "</H1>");
			ib.doTemperatureConvert( state);
			double cTemp = ib.getTemperature( state);
			pw.print( "<H2>");
			pw.print( String.valueOf( cTemp));
			pw.print( " &deg;C (");
			pw.print( String.valueOf( ib.convertToFahrenheit( cTemp)));
			pw.print( " &deg;F)</H2>");
		}
		else
		{
			pw.println( "<H1>1-Wire Device Not Found</H1>");
			pw.print( "1-Wire device ");
			pw.print( ib.getAddressAsString());
			pw.print( " is not present on adapter '");
			pw.print( pa.getAdapterName());
			pw.print( "' on port '");
			pw.print( pa.getPortName());
			pw.println( "'.");
		}
	}
}
