// PortAdapterEnumerator.java - A 1-Wire Port/Adapter enumerator utility class.
//
// Copyright (C) 1999-2002  Smart Software Consulting
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
//
// http://www.smartsc.com
//

package com.smartsc.onewire;

import java.util.Enumeration;

import com.dalsemi.onewire.OneWireAccessProvider;
import com.dalsemi.onewire.OneWireException;
import com.dalsemi.onewire.adapter.DSPortAdapter;

public class PortAdapterEnumerator
{
	public interface Callback
	{
		public void forEachPortAdapter( DSPortAdapter pa, String p);
	}

	public static int enumerate( PortAdapterEnumerator.Callback callback)
	{
		int portAdapterCount = 0;

		Enumeration paEnum = OneWireAccessProvider.enumerateAllAdapters();

		while( paEnum.hasMoreElements())
		{
			DSPortAdapter pa = (DSPortAdapter)paEnum.nextElement();
			Enumeration pEnum = pa.getPortNames();
			while( pEnum.hasMoreElements())
			{
				String p = (String)pEnum.nextElement();
				try
				{
					if( pa.selectPort( p) && pa.adapterDetected())
					{
						callback.forEachPortAdapter( pa, p);
						++portAdapterCount;
					}
				}
				catch( OneWireException ibe) {}
			}
		}

		return portAdapterCount;
	}
}
